unit Joy;

// v1.0
// by Ralf Steinhaeusser in 1998 - ralf@sbox.tu-graz.ac.at

// for help over programming the joystick look in the
// helpfile MM.hlp in the Delphi3/Help - folder

// the whole component can be switched on/off by setting Enabled on/off

// Place Callback-function like this in target-file :
//
// procedure TfrmMain.CallbackJoy(buttons : word;ActionX, ActionY : real);
//
// and connect it like this...
// frmJoy.CallbackFunction:=CallbackJoy;

// don't forget to put "uses MMSystem" into the the file if you want to 
// check the buttons like shown in the example

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, MMSystem, ExtCtrls, Buttons;

type TJoyConfig = record
     maxJoyvalX,
     maxJoyValY : Word;
     CentXMin,CentXMax,
     CentYMin,CentYMax : word;
end;

// Callback-function for reporting Joystick-movement
type TCallbackJoy = procedure (buttons:word;ActionX,ActionY : real) of object;

type
  TfrmJoy = class(TForm)
    Timer1: TTimer;
    GroupBox1: TGroupBox;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    lblPos: TLabel;
    GroupBox2: TGroupBox;
    vertTrackBar: TTrackBar;
    horzTrackbar: TTrackBar;
    GroupBox3: TGroupBox;
    butNext: TButton;
    butCustomize: TButton;
    butOK: TBitBtn;
    butCancel: TBitBtn;
    Panel1: TPanel;
    Memo1: TMemo;
    chkCenter: TCheckBox;
    // Form-calls - Calls from buttons
    procedure butNextClick(Sender: TObject);
    // Form-calls - other events like create, MouseMove ...
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    // Additional stuff
    procedure IdleProc(Sender: TObject; var Done : Boolean);
  private
    { Private declarations }
    status : byte;
  public
    { Public declarations }
    CallbackFunction : TCallbackJoy;   // Pointer to function that has to be called
                                       // when Joy "out of normal"
    Enabled : boolean;

    JoyCfg : TJoyConfig;       // holds "unsensible area", maxX, maxY etc...
  end;

var
  frmJoy: TfrmJoy;

implementation

{$R *.DFM}

// status : 0
const SIntro = 'Press Next-Button to calibrate your Joystick, or customize '+
               'the Joystick-firebuttons. Cancel returns you to Main-program';
// status : 1
const SMov = 'Move the Joystick as far as possible into every direction '+
             'release it and then press the next-button';
// status : 2
const SSensib = 'To avoid over-sensibility for Joystick-action we will now '+
                'determine an "unsensible range" of Joystick-movement.'+
                'Jiggle the Joystick round the center-position and press the '+
                'Next-Button';
// status : 3
const SFin = 'The Joystick is now configured completely.'+
             'Press OK to save configuration or Cancel to abandon it';

procedure TfrmJoy.Timer1Timer(Sender: TObject);
// update the text-label
// determine Joy-position
// call the Callback-function if defined
var
  myjoy: tjoyinfo;
  posX,posY : Word;
  ActionX,ActionY : real;
  butt:word;
begin
     joygetpos(joystickid1,@myjoy);
     posX := myjoy.wxpos;
     posY := myjoy.wypos;
     lblPos.Caption:='Position : '+Format('[%5d,%5d]',[posX,posY]);

     if (@CallbackFunction=nil) or not Enabled then Exit;

     // ############################

     with JoyCfg do
     begin

     {  determine which direction the Joystick is heading  }
     //       2
     //     0 1 2
     //       0

     // first up(2)/down(0)
     if (posy<CentYMin) then                  // Joy up
         ActionY:=2 - posy/CentYMin           // Value between 1 and 2
     else
     if (posy>CentYMax) then                  // Joy down
         ActionY:=1-(posy-CentYMax)/(MaxJoyValY-CentYMax)   // Val = 0-1
     else
         ActionY:=1;                              // Joy in Centerpos.

     // now left(0)/right(2)
     if (posx<CentXMin) then                      // Joy left
         ActionX:=posX/CentXMin                   // Value between 0 and 1
     else
     if (posX>CentXMax) then                      // Joy down
         ActionX:=1+(posx-CentXMax)/(MaxJoyValX-CentXMax)   // Val = 1-2
     else
         ActionX:=1;                              // Joy in Centerpos.

     end;   // End of "with joy do"


     butt:=myjoy.wbuttons;
     CallbackFunction(butt,ActionX,ActionY);

     CallbackFunction(butt,ActionX,ActionY);

     // To check Joystick-buttons :
     // but1 := (myjoy.wbuttons and joy_button1)>0;
     // but2 := (myjoy.wbuttons and joy_button2)>0;
end;

procedure TfrmJoy.IdleProc(Sender: TObject; var Done : Boolean);
// just used with the Calibration-Window
var
  myjoy: tjoyinfo;
  posX,posY : Word;
begin
     joygetpos(joystickid1,@myjoy);
     posX := myjoy.wxpos;
     posY := myjoy.wypos;

     with JoyCfg do
     begin
          if maxJoyvalX=0 then maxJoyvalX:=1;
          if maxJoyvalY=0 then maxJoyvalY:=1;

          // Determine Mexrange of Joystick
          if posx>maxJoyvalX then maxJoyvalX:=posX;
          if posy>maxJoyvalY then maxJoyvalY:=posY;

          if status=2 then
          begin
               //determine dead spot of Joystick
               if posx>CentXMax then CentXMax:=posX;
               if posx<CentXMin then CentXMin:=posX;
               if posy>CentYMax then CentYMax:=posY;
               if posy<CentYMin then CentYMin:=posY;
          end;

          horzTrackBar.Position:=round(horzTrackBar.Max*(posX/maxJoyvalX));
          vertTrackBar.Position:=round(vertTrackBar.Max*(posY/maxJoyvalY));

          checkBox1.checked := (myjoy.wbuttons and joy_button1)>0;
          checkBox2.checked := (myjoy.wbuttons and joy_button2)>0;

          chkCenter.checked:= (posx>CentXMax) or (posx<CentXMin) or
                              (posy>CentYMax) or (posy<CentYMin);

     end;

     Done:=False;
end;


procedure TfrmJoy.butNextClick(Sender: TObject);
var
  myjoy: tjoyinfo;
begin
     with JoyCfg do
     case Status of
     0 : begin                         // Range feststellen
         Memo1.Text:=SMov;
         butOK.Enabled:=False;
         end;
     1 : begin                         // Deadspot feststellen
         joygetpos(joystickid1,@myjoy);
         CentXMax := myjoy.wxpos;
         CentXMin := CentXMax;
         CentYMax := myjoy.wypos;
         CentYMin := CentYMax;

         Memo1.Text:=SSensib;
         end;
     2 : begin                         // Fertig
         Memo1.Text:=SFin;
         butOK.Enabled:=true;
         butOK.ModalResult:=mrOK;
         butNext.Enabled:=false;
         end;
     end;  // Case-End

     Inc(Status);
end;


procedure TfrmJoy.FormClose(Sender: TObject; var Action: TCloseAction);
begin
     Application.OnIdle:=nil;
end;


procedure TfrmJoy.FormShow(Sender: TObject);
begin
     Application.OnIdle:=IdleProc;
     status:=0;
     Memo1.Text:=SIntro;
     butNext.Enabled:=True;
     butOK.Enabled:=True;
end;


procedure TfrmJoy.FormCreate(Sender: TObject);
begin
     Enabled:=False;
     CallbackFunction:=nil;

     with JoyCfg do  // initial configuration
     begin
          CentXMin:=1;
          CentXMax:=2;
          maxJoyvalX:=3;
          CentYMin:=1;
          CentYMax:=2;
          maxJoyValY:=3;
     end;

     butOK.Modalresult:=mrCancel;
end;


end.
